@@ -2,8 +2,6 @@
 Tool for parsing and writing OFF library files to and from dictionaries of
 ResidueTemplate objects
 """
-from __future__ import print_function
-
 from collections import OrderedDict
 from contextlib import closing
 import numpy as np
@@ -15,13 +13,10 @@ from ..modeller.residue import ResidueTemplate, ResidueTemplateContainer
 from ..modeller.residue import PROTEIN, NUCLEIC, SOLVENT, UNKNOWN
 from .. import periodic_table as pt
 from ..utils.io import genopen
-from ..utils.six import string_types, add_metaclass
-from ..utils.six.moves import range
 import re
 import warnings
 
-@add_metaclass(FileFormatType)
-class AmberOFFLibrary(object):
+class AmberOFFLibrary(metaclass=FileFormatType):
     """
     Class containing static methods responsible for parsing and writing OFF
     libraries
@@ -62,8 +57,8 @@ class AmberOFFLibrary(object):
 
     #===================================================
 
-    @staticmethod
-    def id_format(filename):
+    @classmethod
+    def id_format(cls, filename):
         """ Sees if an open file is an OFF library file.
 
         Parameters
@@ -77,14 +72,14 @@ class AmberOFFLibrary(object):
             True if it is recognized as OFF, False otherwise
         """
         with closing(genopen(filename, 'r')) as f:
-            if AmberOFFLibrary._headerre.match(f.readline()):
+            if cls._headerre.match(f.readline()):
                 return True
             return False
 
     #===================================================
 
-    @staticmethod
-    def parse(filename):
+    @classmethod
+    def parse(cls, filename):
         """ Parses an Amber OFF library
 
         Parameters
@@ -109,7 +104,7 @@ class AmberOFFLibrary(object):
         RuntimeError if EOF is reached prematurely or other formatting issues
         found
         """
-        if isinstance(filename, string_types):
+        if isinstance(filename, str):
             fileobj = genopen(filename, 'r')
             own_handle = True
         else:
@@ -117,18 +112,18 @@ class AmberOFFLibrary(object):
             own_handle = False
         # Now parse the library file
         line = fileobj.readline()
-        if not AmberOFFLibrary._headerre.match(line):
+        if not cls._headerre.match(line):
             raise ValueError('Unrecognized OFF file format')
         # Build the return value
         residues = OrderedDict()
         # Pull a list of all the residues we expect to find
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._resre.match(line)
+        rematch = cls._resre.match(line)
         while rematch and line:
             name = rematch.groups()[0]
             residues[name] = None
             line = fileobj.readline()
-            rematch = AmberOFFLibrary._resre.match(line)
+            rematch = cls._resre.match(line)
         if not line:
             raise RuntimeError('Unexpected EOF in Amber OFF library')
         # Now make sure we have the next expected line
@@ -136,21 +131,22 @@ class AmberOFFLibrary(object):
             if not line.strip():
                 line = fileobj.readline()
                 continue
-            rematch = AmberOFFLibrary._sec1re.match(line)
+            rematch = cls._sec1re.match(line)
             if not rematch:
                 raise RuntimeError('Expected atoms table not found')
             name = rematch.groups()[0]
-            residues[name] = AmberOFFLibrary._parse_residue(fileobj, name)
+            residues[name] = cls._parse_residue(fileobj, name)
             line = fileobj.readline()
 
-        if own_handle: fileobj.close()
+        if own_handle:
+            fileobj.close()
 
         return residues
 
     #===================================================
 
-    @staticmethod
-    def _parse_residue(fileobj, name):
+    @classmethod
+    def _parse_residue(cls, fileobj, name):
         """
         Parses the residue information out of the OFF file assuming the file
         is pointed at the first line of an atoms table section of the OFF file
@@ -195,12 +191,13 @@ class AmberOFFLibrary(object):
                 start_atoms.append(runsum)
                 runsum += len(res)
         # Make sure we get the next section
-        rematch = AmberOFFLibrary._sec2re.match(line)
+        rematch = cls._sec2re.match(line)
         if not rematch:
             raise RuntimeError('Expected pertinfo table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f'Found residue {rematch.groups()[0]} while processing residue {name}'
+            )
         line = fileobj.readline()
         while line[0] != '!':
             if not line:
@@ -208,12 +205,13 @@ class AmberOFFLibrary(object):
             # Not used, just skip
             # TODO sanity check
             line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec3re.match(line)
+        rematch = cls._sec3re.match(line)
         if not rematch:
             raise RuntimeError('Expected boundbox table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f'Found residue {rematch.groups()[0]} while processing residue {name}'
+            )
         # Only 5 lines
         try:
             hasbox = float(fileobj.readline().strip())
@@ -231,27 +229,30 @@ class AmberOFFLibrary(object):
                 container.box = [a, b, c, angle, angle, angle]
         # Get the child sequence entry
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec4re.match(line)
+        rematch = cls._sec4re.match(line)
         if not rematch:
             raise RuntimeError('Expected childsequence table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         n = int(fileobj.readline().strip())
         if nres + 1 != n:
-            warnings.warn('Unexpected childsequence (%d); expected %d for '
-                          'residue %s' % (n, nres+1, name), AmberWarning)
+            warnings.warn(
+                f"Unexpected childsequence ({n}); expected {nres+1} for residue {name}",
+                AmberWarning,
+            )
         elif not isinstance(templ, ResidueTemplate) and n != len(templ) + 1:
-            raise RuntimeError('child sequence must be 1 greater than the '
-                               'number of residues in the unit')
+            raise RuntimeError("child sequence must be # of residues in the unit + 1")
         # Get the CONNECT array to set head and tail
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec5re.match(line)
+        rematch = cls._sec5re.match(line)
         if not rematch:
             raise RuntimeError('Expected connect array not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         try:
             head = int(fileobj.readline().strip())
             tail = int(fileobj.readline().strip())
@@ -266,17 +267,19 @@ class AmberOFFLibrary(object):
             templ.tail = templ[tail-1]
         elif tail > 0 and nres > 1:
             if tail < sum((len(r) for r in container)):
-                warnings.warn('TAIL on multi-residue unit not supported (%s). '
-                              'Ignored...' % name, AmberWarning)
+                warnings.warn(
+                    f'TAIL on multi-residue unit not supported ({name}). Ignored...', AmberWarning
+                )
         # Get the connectivity array to set bonds
         line = fileobj.readline()
         if len(templ.atoms) > 1:
-            rematch = AmberOFFLibrary._sec6re.match(line)
+            rematch = cls._sec6re.match(line)
             if not rematch:
                 raise RuntimeError('Expected connectivity table not found')
             elif rematch.groups()[0] != name:
-                raise RuntimeError('Found residue %s while processing residue %s' %
-                                   (rematch.groups()[0], name))
+                raise RuntimeError(
+                    f"Found residue {rematch.groups()[0]} while processing residue {name}"
+                )
             line = fileobj.readline()
             while line[0] != '!':
                 i, j, flag = line.split()
@@ -294,45 +297,49 @@ class AmberOFFLibrary(object):
                 else:
                     templ.add_bond(int(i)-1, int(j)-1)
         # Get the hierarchy table
-        rematch = AmberOFFLibrary._sec7re.match(line)
+        rematch = cls._sec7re.match(line)
         if not rematch:
             raise RuntimeError('Expected hierarchy table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         line = fileobj.readline()
         while line[0] != '!':
             # Skip this section... not used
             # TODO turn this into a sanity check
             line = fileobj.readline()
         # Get the unit name
-        rematch = AmberOFFLibrary._sec8re.match(line)
+        rematch = cls._sec8re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit name string not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         fileobj.readline() # Skip this... not used
         line = fileobj.readline()
         # Get the atomic positions
-        rematch = AmberOFFLibrary._sec9re.match(line)
+        rematch = cls._sec9re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit positions table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         for res in container:
             for atom in res:
                 x, y, z = fileobj.readline().split()
                 atom.xx, atom.xy, atom.xz = float(x), float(y), float(z)
         line = fileobj.readline()
         # Get the residueconnect table
-        rematch = AmberOFFLibrary._sec10re.match(line)
+        rematch = cls._sec10re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit residueconnect table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         for i in range(nres):
             c1,c2,c3,c4,c5,c6 = (int(x) for x in fileobj.readline().split())
             if (c1 > 0 and templ.head is not None and
@@ -346,12 +353,13 @@ class AmberOFFLibrary(object):
                 templ.connections.append(templ[i-1])
         # Get the residues table
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec11re.match(line)
+        rematch = cls._sec11re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit residues table not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         for i in range(nres):
             resname, id, next, start, typ, img = fileobj.readline().split()
             resname = _strip_enveloping_quotes(resname)
@@ -361,9 +369,10 @@ class AmberOFFLibrary(object):
             typ = _strip_enveloping_quotes(typ)
             img = int(img)
             if next - start != len(container[i]):
-                warnings.warn('residue table predicted %d, not %d atoms for '
-                              'residue %s' % (next-start, len(container[i]),
-                              name), AmberWarning)
+                warnings.warn(
+                    f'residue table predicted {next - start}, not {len(container[i])} atoms for residue {name}',
+                    AmberWarning,
+                )
             if typ == 'p':
                 container[i].type = PROTEIN
             elif typ == 'n':
@@ -376,23 +385,25 @@ class AmberOFFLibrary(object):
                 container[i].name = resname
         # Get the residues sequence table
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec12re.match(line)
+        rematch = cls._sec12re.match(line)
         if not rematch:
             raise RuntimeError('Expected residue sequence number not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         for i in range(nres):
             #TODO sanity check
             fileobj.readline()
         line = fileobj.readline()
         # Get the solventcap array
-        rematch = AmberOFFLibrary._sec13re.match(line)
+        rematch = cls._sec13re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit solventcap array not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         # Ignore the solvent cap
         fileobj.readline()
         fileobj.readline()
@@ -401,12 +412,13 @@ class AmberOFFLibrary(object):
         fileobj.readline()
         # Velocities
         line = fileobj.readline()
-        rematch = AmberOFFLibrary._sec14re.match(line)
+        rematch = cls._sec14re.match(line)
         if not rematch:
             raise RuntimeError('Expected unit solventcap array not found')
         elif rematch.groups()[0] != name:
-            raise RuntimeError('Found residue %s while processing residue %s' %
-                               (rematch.groups()[0], name))
+            raise RuntimeError(
+                f"Found residue {rematch.groups()[0]} while processing residue {name}"
+            )
         for res in container:
             for atom in res:
                 vx, vy, vz = (float(x) for x in fileobj.readline().split())
@@ -418,8 +430,8 @@ class AmberOFFLibrary(object):
 
     #===================================================
 
-    @staticmethod
-    def write(lib, dest):
+    @classmethod
+    def write(cls, lib, dest):
         """ Writes a dictionary of ResidueTemplate units to a file in OFF format
 
         Parameters
@@ -440,7 +452,7 @@ class AmberOFFLibrary(object):
         for name in names:
             dest.write(' "%s"\n' % name)
         for name in names:
-            AmberOFFLibrary._write_residue(dest, lib[name])
+            cls._write_residue(dest, lib[name])
 
         if own_handle: dest.close()
 
@@ -593,7 +605,7 @@ def _imaging_atom(res):
     except 1, it is the heavy atom. Otherwise, it is the atom *closest* to the
     COM of the residue
     """
-    from parmed.geometry import center_of_mass
+    from ..geometry import center_of_mass
     #TODO implement the docstring
     found_heavy = False
     heavy_idx = -1
