@@ -2,29 +2,25 @@
 This module contains functionality for running simulations with OpenMM using
 parameters defined in an input file for sander and pmemd.
 """
-from __future__ import division
-
 import logging
+import os
+from math import sqrt
+
 from ...amber import AmberMdcrd, AmberMask, NetCDFTraj, Rst7
 from ...amber.mdin import Mdin
 from ...openmm import (StateDataReporter, NetCDFReporter, MdcrdReporter, RestartReporter,
-                           ProgressReporter, EnergyMinimizerReporter)
+                       ProgressReporter, EnergyMinimizerReporter)
 from ...utils.timer import Timer
 from ... import unit as u
-from ...utils.six.moves import range
-from math import sqrt
-import os
 from ..exceptions import SimulationError, SimulationWarning, UnhandledArgumentWarning
 import sys
 import warnings
+from ...vec3 import Vec3
 try:
-    try:
-        from simtk.openmm.vec3 import Vec3
-    except ImportError:
-        from openmm.vec3 import Vec3
-    from simtk.openmm.app import (forcefield as ff, OBC1, OBC2, GBn, HCT, GBn2,
-                                  Simulation, DCDReporter, amberprmtopfile)
-    import simtk.openmm as mm
+    from openmm.app import (
+        forcefield as ff, OBC1, OBC2, GBn, HCT, GBn2, Simulation, DCDReporter, amberprmtopfile
+    )
+    import openmm as mm
     HAS_OPENMM = True
 except ImportError:
     HAS_OPENMM = False
@@ -32,21 +28,19 @@ except ImportError:
 LOGGER = logging.getLogger(__name__)
 
 _SCRIPT_HEADER = """\
-#!/usr/bin/env python
-from __future__ import division, print_function
+#!/usr/bin/env python3
 
 import os, sys
 
 # Import the OpenMM modules that are necessary
-from simtk.openmm.app import *
-from simtk.openmm import *
+from openmm.app import *
+from openmm import *
 
 # Import the Amber/OpenMM modules
 from parmed.amber import AmberParm, Rst7, AmberMdcrd, AmberMask, NetCDFTraj
 from parmed.openmm import (StateDataReporter, NetCDFReporter, MdcrdReporter,
         RestartReporter, ProgressReporter, EnergyMinimizerReporter)
 from parmed import unit as u
-from parmed.utils.six.moves import range
 
 # Load the Amber topology file
 parm = AmberParm('%s', '%s')
@@ -59,7 +53,7 @@ def positional_restraints(mask, weights, refc, scriptfile=None):
     """
     parm = mask.parm # store the parm object
     try:
-        refc = refc.reshape((-1, len(parm.atoms), 3))
+        refc = refc.coordinates.reshape((-1, len(parm.atoms), 3))
     except ValueError:
         raise SimulationError('Invalid shape of coordinate array')
 
@@ -568,7 +562,7 @@ def simulate(parm, args):
                                       maxIterations=mdin.cntrl_nml['maxcyc'])
             rep.report(simulation)
             # Write a restart file with the new coordinates
-            restrt_reporter = RestartReporter(restart, 1, parm.ptr('natom'), False,
+            restrt_reporter = RestartReporter(restart, 1, False,
                                               mdin.cntrl_nml['ntxo'] == 2, write_velocities=False)
             restrt_reporter.report(simulation, simulation.context.getState(getPositions=True,
                                    enforcePeriodicBox=bool(mdin.cntrl_nml['ntb'])))
